package de.fitconnect.dependency;

import com.google.inject.AbstractModule;
import com.google.inject.Provides;
import com.google.inject.name.Named;
import com.google.inject.name.Names;
import fitconnect.api.services.Sender;
import fitconnect.api.services.Subscriber;
import fitconnect.api.services.auth.OAuthService;
import fitconnect.api.services.crypto.CryptoService;
import fitconnect.api.services.validation.CertificateValidator;
import fitconnect.api.services.validation.MetadataValidator;
import fitconnect.impl.SubmissionSender;
import fitconnect.impl.SubmissionSubscriber;
import fitconnect.impl.auth.MonoPublisherOAuthService;
import fitconnect.impl.crypto.JWECryptoService;
import fitconnect.impl.validation.KeyValidator;
import fitconnect.impl.validation.MetadataSubmissionValidator;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

public class SdkModule extends AbstractModule {

    @Override
    protected void configure() {

        bindConfigProperties();

        bind(CryptoService.class).to(JWECryptoService.class);
        bind(CertificateValidator.class).to(KeyValidator.class);
        bind(MetadataValidator.class).to(MetadataSubmissionValidator.class);

        bind(Sender.class).to(SubmissionSender.class);
        bind(Subscriber.class).to(SubmissionSubscriber.class);
    }

    @Provides
    OAuthService provideOAuthService(@Named("tokenUrl")  String tokenUrl) {
        return new MonoPublisherOAuthService(tokenUrl);
    }

    private void bindConfigProperties() {
        try {
            final ClassLoader classloader = Thread.currentThread().getContextClassLoader();
            final InputStream is = classloader.getResourceAsStream("sdk.properties");
            final Properties properties = new Properties();
            properties.load(is);
            Names.bindProperties(binder(), properties);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
